package me.teridax.jcash;

import me.teridax.jcash.gui.IconProvider;
import me.teridax.jcash.gui.Loader;
import me.teridax.jcash.gui.account.AccountController;
import me.teridax.jcash.gui.login.LoginController;
import me.teridax.jcash.lang.Locales;

import javax.swing.*;
import java.util.Objects;
import java.util.logging.*;

import static javax.swing.JOptionPane.ERROR_MESSAGE;
import static javax.swing.JOptionPane.showMessageDialog;
import static me.teridax.jcash.Logging.LOGGER;
import static me.teridax.jcash.Logging.initializeSystemLogger;
import static me.teridax.jcash.lang.Translator.translate;

public final class Main {

    /**
     * Main instance of this program. Contains the primary window.
     */
    private static Main instance;

    /**
     * Primary window of this program
     */
    private final JFrame window;

    private Main() {
        // create main window and set defaults
        this.window = new JFrame();
        this.window.setTitle(translate("Cashmachine"));
        this.window.setLocationByPlatform(true);
        this.window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.window.setIconImage(IconProvider.getWindowIcon());
    }

    public static void main(String[] args) {
        initializeSystemLogger(Level.FINE);

        Locales.autodetectDefaultLocale();

        setPlatformDependingTheme();

        // create main instance and show the login screen
        instance();
        getInstance().showLoginScreen();
    }

    /**
     * Set the look and feel via the ui manager.
     * This function will select a look and feel so that the application will
     * look most like a native application.
     * It will select the systems look and feel for Windows and MacOS
     * and GTK for unix based systems.
     */
    private static void setPlatformDependingTheme() {
        // default look and feel
        var laf = UIManager.getCrossPlatformLookAndFeelClassName();

        // runtime os
        String os = System.getProperty("os.name").toLowerCase();
        LOGGER.fine("Detected operating system: " + os);

        // set look and feel class name depending on the platform we run on
        if (os.contains("win") || os.contains("mac")) {
            LOGGER.info("Detected Windows or MacOS based operating system, using system look and feel");
            laf = UIManager.getSystemLookAndFeelClassName();
        } else if (os.contains("nix") || os.contains("nux") || os.contains("aix") || os.contains("sunos")) {
            LOGGER.info("Detected Unix/Linux based operating system, using GTK look and feel");
            laf = "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
        } else {
            LOGGER.warning("Unable to detect operating system: " + os);
        }

        try {
            LOGGER.info("Setting look and feel to class: " + laf);
            UIManager.setLookAndFeel(laf);
        } catch (ClassNotFoundException | UnsupportedLookAndFeelException e) {
            LOGGER.severe("Look and feel class not found: " + e.getMessage());
        } catch (InstantiationException | IllegalAccessException e) {
            LOGGER.warning("Could not set look and feel: " + e.getMessage());
        }
    }

    /**
     * Get the main singleton instance of this program
     * @return the singleton instance of this class
     */
    public static Main getInstance() {
        return instance;
    }

    /**
     * Attempts to create a new instance of the singleton class Main.
     * This method throws an exception if the class is already instantiated.
     * @throws IllegalStateException if the class is already instantiated
     */
    public static void instance() {
        if (null != instance)
            throw new IllegalStateException(Main.class.getName() + " is already initialized");

        LOGGER.fine("Creating singleton instance of class " + Main.class.getName());

        Main.instance = new Main();
    }

    /**
     * Shows the open dialog for selecting a database file. After selection, it then proceeds to prompt login.
     * Afterward the selected account can be managed.
     * This method is non-blocking and all work described is performed asynchronously on the AWT Event dispatcher.
     */
    public void showLoginScreen() {
        SwingUtilities.invokeLater(() -> {
            LOGGER.finer("showing login screen");

            try {
                // select db file
                var path = Loader.load();
                // read database and login
                var login = new LoginController(path);

                // when we have logged in set the account viewer as window content
                login.addAccountSelectionListener(account -> {
                    LOGGER.finer("account selected: " + Objects.toString(account, "null"));

                    var profileCont = new AccountController(account, login.getData().getBms());
                    this.window.setContentPane(profileCont.getView());
                    this.window.revalidate();
                    this.window.repaint();
                });

                // we are not logged in yet, so show the login prompt on the main window
                this.window.setContentPane(login.getView());
                this.window.setSize(800, 600);
                this.window.setVisible(true);

            } catch (IllegalStateException e) {
                LOGGER.fine("Unable to show login mask: " + e.getMessage());
                showMessageDialog(null, e.getMessage(), translate("Closing JCash"), ERROR_MESSAGE);
                System.exit(0);
            }
        });
    }

    /**
     * Logs the user out of the database, hiding the main window.
     */
    public void logout() {
        window.setContentPane(new JLabel(translate("you're logged out")));
        window.setVisible(false);
    }

    public JFrame getWindow() {
        return this.window;
    }
}